import collections
import re

_KNOWN_OPERATION_TYPES = (
    "BATCH.DELETE.OBJECT",
    "REST.COPY.OBJECT",
    "REST.COPY.OBJECT_GET",
    "REST.COPY.PART",
    "REST.DELETE.BUCKET",
    "REST.DELETE.BUCKETPOLICY",
    "REST.DELETE.LIFECYCLE",
    "REST.DELETE.OBJECT",
    "REST.DELETE.OBJECT_TAGGING",
    "REST.DELETE.UPLOAD",
    "REST.GET.ACCELERATE",
    "REST.GET.ACL",
    "REST.GET.ANALYTICS",
    "REST.GET.BUCKET",
    "REST.GET.BUCKETPOLICY",
    "REST.GET.BUCKETVERSIONS",
    "REST.GET.CORS",
    "REST.GET.ENCRYPTION",
    "REST.GET.INTELLIGENT_TIERING",
    "REST.GET.INVENTORY",
    "REST.GET.LIFECYCLE",
    "REST.GET.LOCATION",
    "REST.GET.LOGGING_STATUS",
    "REST.GET.METRICS",
    "REST.GET.NOTIFICATION",
    "REST.GET.OBJECT",
    "REST.GET.OBJECT_LOCK_CONFIGURATION",
    "REST.GET.OBJECT_TAGGING",
    "REST.GET.OWNERSHIP_CONTROLS",
    "REST.GET.PART",
    "REST.GET.POLICY_STATUS",
    "REST.GET.PUBLIC_ACCESS_BLOCK",
    "REST.GET.REPLICATION",
    "REST.GET.REQUEST_PAYMENT",
    "REST.GET.TAGGING",
    "REST.GET.UPLOAD",
    "REST.GET.UPLOADS",
    "REST.GET.VERSIONING",
    "REST.GET.WEBSITE",
    "REST.HEAD.BUCKET",
    "REST.HEAD.BUCKETVERSIONS",
    "REST.HEAD.OBJECT",
    "REST.HEAD.PART",
    "REST.HEAD.UPLOADS",
    "REST.OPTIONS.PREFLIGHT",
    "REST.POST.BUCKET",
    "REST.POST.MULTI_OBJECT_DELETE",
    "REST.POST.OBJECT",
    "REST.POST.UPLOAD",
    "REST.POST.UPLOADS",
    "REST.PUT.ACL",
    "REST.PUT.BUCKET",
    "REST.PUT.BUCKETPOLICY",
    "REST.PUT.CORS",
    "REST.PUT.INVENTORY",
    "REST.PUT.LIFECYCLE",
    "REST.PUT.LOGGING_STATUS",
    "REST.PUT.METRICS",
    "REST.PUT.NOTIFICATION",
    "REST.PUT.OBJECT",
    "REST.PUT.OWNERSHIP_CONTROLS",
    "REST.PUT.PART",
    "REST.PUT.VERSIONING",
    "WEBSITE.GET.OBJECT",
    # "objects;"  # Unsure about this last one; it showed up in a scan of all 7-th string elements
)

_IS_OPERATION_TYPE_KNOWN = collections.defaultdict(bool)
for request_type in _KNOWN_OPERATION_TYPES:
    _IS_OPERATION_TYPE_KNOWN[request_type] = True

_S3_LOG_FIELDS = (
    "bucket_owner",
    "bucket",
    "timestamp",
    "ip_address",
    "requester",
    "request_id",
    "operation",
    "object_key",
    "request_uri",
    # "http_version",  # Regex not splitting this from the request_uri...
    "http_status_code",
    "error_code",
    "bytes_sent",
    "object_size",
    "total_time",
    "turn_around_time",
    "referrer",
    "user_agent",
    "version_id",
    "host_id",
    "signature_version",
    "cipher_suite",
    "authentication_type",
    "host_header",
    "tls_version",
    "access_point_arn",
    "acl_required",
)
_FullLogLine = collections.namedtuple("FullLogLine", _S3_LOG_FIELDS)

_S3_LOG_REGEX = re.compile(pattern=r'"([^"]+)"|\[([^]]+)]|([^ ]+)')

_KNOWN_SERVICES = ("GitHub", "AWS", "GCP", "VPN")  # Azure has problems; see _ip_utils.py for more info
